/*
 * xrlist.c
 *
 *  $Id:$
 */

#define VERSION "1.10"

#include <unistd.h>
#include <stdio.h>
#include <rpcsvc/rstat.h>
#include <X11/StringDefs.h>
#include <X11/Intrinsic.h>
#include <X11/Xaw/Form.h>
#include <X11/Xaw/Command.h>
#include <X11/Xaw/Label.h>
#include <X11/Xaw/StripChart.h>

#define	Xt_W	XtVaCreateManagedWidget

#define	DEFAULT_INTERVAL  60  /* 1min. */
#define	STRING_MAX        256
#define	HOST_MAX          8

Widget top,base,quit,label[HOST_MAX],chart[HOST_MAX];

struct HostInfo {
    Widget wl,wv,wc;
    double value;
    char name[9+1];
} host[HOST_MAX];

/*
 * ǥե ꥽ǡ
 */
static char *fallback_resources[] = {
"*font: 6x12",
"*borderColor: #c0c0d0",
"*background: #101020",
"*foreground: #c0c0d0",
"*StripChart*foreground: #ffff80",
"*StripChart*width: 48",
"*StripChart*height: 36",
  NULL
};

void Quit();
void GetValue();

void main(int argc,char **argv)
{
	XtAppContext app_context;
	int i,c;
	int quit_f = FALSE;
	int host_max = 0;
	int interval = DEFAULT_INTERVAL;
	char name[64];

	extern char *optarg;
	extern int optind;
	char *intstr;

	/*
	 * Xt Initialize
	 */
	top = XtVaAppInitialize(&app_context,"Xrlist"
			        ,NULL,0
				,(Cardinal *)&argc,argv,fallback_resources,NULL);

	/*
	 * Command analysis
	 */
	i = 1;
	while(--argc) {
	    if (*argv[i] == '-') {
		switch(*(argv[i]+1)) {
		    case 's':
			interval = atoi(argv[i]+2);
			if (interval <= 0)
			    interval = DEFAULT_INTERVAL;
			break;
		    case 'q':
			quit_f = TRUE;
			break;
		    default:
			fprintf(stderr,"Xrlist v%s  1997 by Kunihiko Hayashi\n"
			               "usage: %s [-s<interval>][-q] hosts...\n",VERSION,argv[0]);
			exit(2);
		}
	    }
	    else if (host_max < HOST_MAX) {
		strncpy(host[host_max].name,argv[i],9);
		host_max++;	
	    }
	    i++;
	}

	if (host_max == 0) {
		strncpy(host[0].name,"localhost\0",10);
		host_max++;
	}

	/*
	 * make widgets
	 */
	base = Xt_W("base",formWidgetClass,top,NULL);

	if (quit_f) {
	  quit = Xt_W("quit",commandWidgetClass,base,
		      XtNlabel,"Quit",
		      NULL);
	}

	for(i=0;i<host_max;i++) {
		sprintf(name,"l%d",i);
		host[i].wl = Xt_W(name,labelWidgetClass,base,
				  XtNfromVert,(i==0)?((quit_f)?quit:NULL):host[i-1].wc,
				  XtNborderWidth,0,
				  NULL);

		*name ='v';
		host[i].wv = Xt_W(name,labelWidgetClass,base,
				  XtNfromVert,host[i].wl,
				  XtNborderWidth,2,
				  NULL);

		*name = 'c';
		host[i].wc = Xt_W(name,stripChartWidgetClass,base,
				  XtNjumpScroll,1,
				  XtNupdate,interval,	/* sec */
				  XtNfromHoriz,host[i].wv,
				  XtNfromVert,(i==0)?quit:host[i-1].wc,
				  NULL);

		XtVaSetValues(host[i].wl,XtNlabel,host[i].name,NULL);
		XtVaSetValues(host[i].wv,XtNlabel,"    -.--",NULL);
		XtAddCallback(host[i].wc,XtNgetValue,GetValue,&host[i]);
	}

	if (quit_f)
	 XtAddCallback(quit,XtNcallback,Quit,NULL);

	XtRealizeWidget(top);
	XtAppMainLoop(app_context);
}

/*
 * Exit (Quit action)
 */
void Quit(Widget w)
{
	exit(0);
}

/*
 * Get value (Chart Update action)
 */
void GetValue(Widget w,struct HostInfo *rhost,double *value)
{
	char str[10];
	float v = 0.00;
	struct statstime stat;

	rstat(rhost->name,&stat);
	v = (double)stat.avenrun[0]/256.0;

	sprintf(str,"%8.2f",v);
	XtVaSetValues(rhost->wv,XtNlabel,str,NULL);

	rhost->value = v;  
	*value       = v;
}
